import { Express, Request, Response } from 'express';
import { createServer, type Server } from 'http';
import path from 'path';
import { setupAuth } from './auth';
import { storage } from './storage';
import { movies, series, channels, matches, categories, subcategories, notifications, siteSettings } from '@shared/schema';
import { eq } from 'drizzle-orm';
import { db } from './db';
import multer from 'multer';
import fs from 'fs';

const upload = multer({
  dest: './uploads/',
  limits: { fileSize: 100 * 1024 * 1024 }, // 100MB file size limit
});

export async function registerRoutes(app: Express): Promise<Server> {
  // Auth routes
  setupAuth(app);

  // Make deployment directory accessible for downloads
  app.use('/downloads', express.static(path.join(process.cwd(), 'deployment')));

  // Add download links page
  app.get('/download-center', (req, res) => {
    res.send(`
      <!DOCTYPE html>
      <html dir="rtl">
      <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>مركز تحميل صقر تي في</title>
        <style>
          body {
            font-family: 'Segoe UI', Tahoma, Geneva, Verdana, sans-serif;
            margin: 0;
            padding: 0;
            background-color: #f5f5f5;
            direction: rtl;
          }
          .container {
            max-width: 800px;
            margin: 50px auto;
            padding: 30px;
            background-color: white;
            border-radius: 10px;
            box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
          }
          h1 {
            color: #0050b3;
            text-align: center;
            margin-bottom: 30px;
          }
          .download-item {
            margin-bottom: 20px;
            padding: 20px;
            border: 1px solid #e8e8e8;
            border-radius: 6px;
            background-color: #fafafa;
            transition: all 0.3s ease;
          }
          .download-item:hover {
            border-color: #1890ff;
            box-shadow: 0 2px 8px rgba(0, 0, 0, 0.15);
          }
          h2 {
            color: #262626;
            margin-top: 0;
          }
          p {
            color: #595959;
            margin-bottom: 15px;
          }
          .download-button {
            display: inline-block;
            padding: 8px 16px;
            background-color: #1890ff;
            color: white;
            text-decoration: none;
            border-radius: 4px;
            transition: background-color 0.3s ease;
          }
          .download-button:hover {
            background-color: #40a9ff;
          }
          .file-info {
            margin-top: 10px;
            font-size: 14px;
            color: #8c8c8c;
          }
        </style>
      </head>
      <body>
        <div class="container">
          <h1>مركز تحميل صقر تي في</h1>
          
          <div class="download-item">
            <h2>الحزمة الكاملة لمنصة صقر تي في</h2>
            <p>تحميل الحزمة الكاملة التي تشمل موقع الويب وتطبيق صقر تي في وتطبيق مشغل صقر</p>
            <a href="/downloads/saqr-tv-full-package.zip" class="download-button">تحميل الحزمة الكاملة</a>
            <div class="file-info">حجم الملف: ~900 كيلوبايت</div>
          </div>
          
          <div class="download-item">
            <h2>موقع وخادم صقر تي في</h2>
            <p>تحميل موقع الويب الخاص بصقر تي في ولوحة الإدارة</p>
            <a href="/downloads/saqr-tv-backend.zip" class="download-button">تحميل موقع الويب</a>
            <div class="file-info">حجم الملف: ~240 كيلوبايت</div>
          </div>
          
          <div class="download-item">
            <h2>تطبيق صقر تي في للجوال</h2>
            <p>تحميل تطبيق صقر تي في للهواتف المحمولة</p>
            <a href="/downloads/saqr-tv-mobile-app.zip" class="download-button">تحميل تطبيق الجوال</a>
            <div class="file-info">حجم الملف: ~180 كيلوبايت</div>
          </div>
          
          <div class="download-item">
            <h2>تطبيق مشغل صقر</h2>
            <p>تحميل تطبيق مشغل صقر مع دعم الإعلانات من AdMob</p>
            <a href="/downloads/saqr-player.zip" class="download-button">تحميل المشغل</a>
            <div class="file-info">حجم الملف: ~14 كيلوبايت</div>
          </div>
          
          <div class="download-item">
            <h2>نصوص التثبيت والإعداد</h2>
            <p>تحميل نصوص التثبيت وأدلة المستخدم</p>
            <a href="/downloads/download-mobile-app.sh" class="download-button">نص تحميل تطبيق الجوال</a>
            <a href="/downloads/download-player-app.sh" class="download-button" style="margin-right: 10px;">نص تحميل المشغل</a>
          </div>
        </div>
      </body>
      </html>
    `);
  });

  // Settings routes
  app.get('/api/settings', async (req, res) => {
    const settings = await storage.getSettings();
    res.json(settings);
  });

  app.patch('/api/settings', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const settings = await storage.updateSettings(req.body);
    res.json(settings);
  });

  // Movie routes
  app.get('/api/movies', async (req, res) => {
    const movies = await storage.getMovies();
    res.json(movies);
  });

  app.get('/api/movies/:id', async (req, res) => {
    const id = parseInt(req.params.id);
    const movie = await storage.getMovie(id);
    if (!movie) return res.status(404).send('Movie not found');
    res.json(movie);
  });

  app.post('/api/movies', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const movie = await storage.createMovie(req.body);
    res.status(201).json(movie);
  });

  app.patch('/api/movies/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const movie = await storage.updateMovie(id, req.body);
    if (!movie) return res.status(404).send('Movie not found');
    res.json(movie);
  });

  app.delete('/api/movies/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const success = await storage.deleteMovie(id);
    if (!success) return res.status(404).send('Movie not found');
    res.sendStatus(204);
  });

  // Series routes
  app.get('/api/series', async (req, res) => {
    const seriesList = await storage.getAllSeries();
    res.json(seriesList);
  });

  app.get('/api/series/:id', async (req, res) => {
    const id = parseInt(req.params.id);
    const series = await storage.getSeries(id);
    if (!series) return res.status(404).send('Series not found');
    res.json(series);
  });

  app.post('/api/series', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const series = await storage.createSeries(req.body);
    res.status(201).json(series);
  });

  app.patch('/api/series/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const series = await storage.updateSeries(id, req.body);
    if (!series) return res.status(404).send('Series not found');
    res.json(series);
  });

  app.delete('/api/series/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const success = await storage.deleteSeries(id);
    if (!success) return res.status(404).send('Series not found');
    res.sendStatus(204);
  });

  // Episode routes
  app.get('/api/series/:seriesId/episodes', async (req, res) => {
    const seriesId = parseInt(req.params.seriesId);
    const episodes = await storage.getEpisodes(seriesId);
    res.json(episodes);
  });

  app.get('/api/episodes/:id', async (req, res) => {
    const id = parseInt(req.params.id);
    const episode = await storage.getEpisode(id);
    if (!episode) return res.status(404).send('Episode not found');
    res.json(episode);
  });

  app.post('/api/episodes', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const episode = await storage.createEpisode(req.body);
    res.status(201).json(episode);
  });

  app.patch('/api/episodes/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const episode = await storage.updateEpisode(id, req.body);
    if (!episode) return res.status(404).send('Episode not found');
    res.json(episode);
  });

  app.delete('/api/episodes/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const success = await storage.deleteEpisode(id);
    if (!success) return res.status(404).send('Episode not found');
    res.sendStatus(204);
  });

  // Channel routes
  app.get('/api/channels', async (req, res) => {
    const channels = await storage.getChannels();
    res.json(channels);
  });

  app.get('/api/channels/:id', async (req, res) => {
    const id = parseInt(req.params.id);
    const channel = await storage.getChannel(id);
    if (!channel) return res.status(404).send('Channel not found');
    res.json(channel);
  });

  app.post('/api/channels', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const channel = await storage.createChannel(req.body);
    res.status(201).json(channel);
  });

  app.patch('/api/channels/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const channel = await storage.updateChannel(id, req.body);
    if (!channel) return res.status(404).send('Channel not found');
    res.json(channel);
  });

  app.delete('/api/channels/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const success = await storage.deleteChannel(id);
    if (!success) return res.status(404).send('Channel not found');
    res.sendStatus(204);
  });

  // Match routes
  app.get('/api/matches', async (req, res) => {
    const matches = await storage.getMatches();
    res.json(matches);
  });

  app.get('/api/matches/:id', async (req, res) => {
    const id = parseInt(req.params.id);
    const match = await storage.getMatch(id);
    if (!match) return res.status(404).send('Match not found');
    res.json(match);
  });

  app.post('/api/matches', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const match = await storage.createMatch(req.body);
    res.status(201).json(match);
  });

  app.patch('/api/matches/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const match = await storage.updateMatch(id, req.body);
    if (!match) return res.status(404).send('Match not found');
    res.json(match);
  });

  app.delete('/api/matches/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const success = await storage.deleteMatch(id);
    if (!success) return res.status(404).send('Match not found');
    res.sendStatus(204);
  });

  // Category routes
  app.get('/api/categories', async (req, res) => {
    const categories = await storage.getCategories();
    res.json(categories);
  });

  app.get('/api/categories/:id', async (req, res) => {
    const id = parseInt(req.params.id);
    const category = await storage.getCategory(id);
    if (!category) return res.status(404).send('Category not found');
    res.json(category);
  });

  app.post('/api/categories', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const category = await storage.createCategory(req.body);
    res.status(201).json(category);
  });

  app.patch('/api/categories/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const category = await storage.updateCategory(id, req.body);
    if (!category) return res.status(404).send('Category not found');
    res.json(category);
  });

  app.delete('/api/categories/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const success = await storage.deleteCategory(id);
    if (!success) return res.status(404).send('Category not found');
    res.sendStatus(204);
  });

  // Subcategory routes
  app.get('/api/subcategories', async (req, res) => {
    const type = req.query.type as string;
    if (type) {
      const subcategories = await storage.getSubcategoriesByType(type);
      return res.json(subcategories);
    }
    
    const parentId = req.query.parentId ? parseInt(req.query.parentId as string) : undefined;
    if (parentId) {
      const subcategories = await storage.getSubcategoriesByParent(parentId);
      return res.json(subcategories);
    }
    
    const subcategories = await storage.getSubcategories();
    res.json(subcategories);
  });

  app.get('/api/subcategories/:id', async (req, res) => {
    const id = parseInt(req.params.id);
    const subcategory = await storage.getSubcategory(id);
    if (!subcategory) return res.status(404).send('Subcategory not found');
    res.json(subcategory);
  });

  app.post('/api/subcategories', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const subcategory = await storage.createSubcategory(req.body);
    res.status(201).json(subcategory);
  });

  app.patch('/api/subcategories/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const subcategory = await storage.updateSubcategory(id, req.body);
    if (!subcategory) return res.status(404).send('Subcategory not found');
    res.json(subcategory);
  });

  app.delete('/api/subcategories/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const success = await storage.deleteSubcategory(id);
    if (!success) return res.status(404).send('Subcategory not found');
    res.sendStatus(204);
  });

  // Notification routes
  app.get('/api/notifications', async (req, res) => {
    const notifications = await storage.getNotifications();
    res.json(notifications);
  });

  app.get('/api/notifications/:id', async (req, res) => {
    const id = parseInt(req.params.id);
    const notification = await storage.getNotification(id);
    if (!notification) return res.status(404).send('Notification not found');
    res.json(notification);
  });

  app.post('/api/notifications', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const notification = await storage.createNotification(req.body);
    res.status(201).json(notification);
  });

  app.patch('/api/notifications/:id/read', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const notification = await storage.markNotificationAsRead(id);
    if (!notification) return res.status(404).send('Notification not found');
    res.json(notification);
  });

  app.delete('/api/notifications/:id', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    const id = parseInt(req.params.id);
    const success = await storage.deleteNotification(id);
    if (!success) return res.status(404).send('Notification not found');
    res.sendStatus(204);
  });
  
  // Recent content route
  app.get('/api/recent-content', async (req, res) => {
    const limit = req.query.limit ? parseInt(req.query.limit as string) : undefined;
    const content = await storage.getRecentContent(limit);
    res.json(content);
  });
  
  // File upload for mobile apps
  app.post('/api/upload-app', upload.single('app'), async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    if (!req.file) return res.status(400).send('No file uploaded');
    
    const appType = req.body.appType as string;
    if (!appType || !['saqr-tv', 'saqr-player'].includes(appType)) {
      return res.status(400).send('Invalid app type');
    }
    
    const appVersion = req.body.appVersion || '1.0.0';
    const fileName = appType === 'saqr-tv' ? 'saqr-tv-app.apk' : 'saqr-player.apk';
    const targetPath = path.join('./uploads', fileName);
    
    try {
      // Move file from temp upload to final destination
      fs.renameSync(req.file.path, targetPath);
      
      // Update app version in database
      const settings = await storage.getSettings();
      const updatedSettings = { ...settings };
      
      if (appType === 'saqr-tv') {
        updatedSettings.appVersion = appVersion;
      } else {
        updatedSettings.playerVersion = appVersion;
      }
      
      await storage.updateSettings(updatedSettings);
      
      res.status(200).json({
        success: true,
        message: `${appType === 'saqr-tv' ? 'Saqr TV' : 'Saqr Player'} app uploaded successfully`,
        version: appVersion,
        downloadUrl: `/uploads/${fileName}`
      });
    } catch (error) {
      res.status(500).json({
        success: false,
        message: 'Error saving file',
        error: (error as Error).message
      });
    }
  });

  // Video link extraction API
  app.post('/api/extract-video', async (req, res) => {
    if (!req.isAuthenticated()) return res.sendStatus(401);
    
    const { sourceUrl } = req.body;
    if (!sourceUrl) return res.status(400).send('Source URL is required');
    
    try {
      // This is a mock implementation - in a real scenario, 
      // you would implement logic to extract video URLs from various sites
      // For demo purposes, we'll just return the source URL
      res.json({
        success: true,
        videoUrl: sourceUrl,
        message: 'Video link extracted successfully'
      });
    } catch (error) {
      res.status(500).json({
        success: false,
        message: 'Failed to extract video link',
        error: (error as Error).message
      });
    }
  });

  const httpServer = createServer(app);
  return httpServer;
}

import express from 'express';